'use strict';

Object.defineProperty(exports, "__esModule", {
  value: true
});

var _compose = require('../compose');

var _compose2 = _interopRequireDefault(_compose);

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

function composeWithTracker(reactiveFn, L, E) {
  var options = arguments.length <= 3 || arguments[3] === undefined ? { displayName: 'WithTracker' } : arguments[3];

  var onPropsChange = function onPropsChange(props, onData) {
    if (!props.context) {
      throw new Error('No context passed as prop.');
    }

    var context = typeof props.context === 'function' ? props.context() : props.context;
    var Tracker = context.Tracker || context.tracker || context.Trackr;

    if (!Tracker) {
      throw new Error('No Tracker found in the context');
    }

    var trackerCleanup = void 0;
    var handler = Tracker.nonreactive(function () {
      return Tracker.autorun(function () {
        trackerCleanup = reactiveFn(props, onData);
      });
    });

    return function () {
      if (typeof trackerCleanup === 'function') {
        trackerCleanup();
      }
      return handler.stop();
    };
  };

  return (0, _compose2.default)(onPropsChange, L, E, options);
}

exports.default = composeWithTracker;